@@ -41,6 +41,7 @@
 #include "prefs.h"
 #include "account.h"
 #include "prefs_account.h"
+#include "sylmain.h"
 
 static GList *folder_list = NULL;
 
@@ -191,6 +192,7 @@ FolderItem *folder_item_new(const gchar *name, const gchar *path)
 	item->auto_replyto = NULL;
 	item->mark_queue = NULL;
 	item->last_selected = 0;
+	item->qsearch_cond_type = 0;
 	item->data = NULL;
 
 	return item;
@@ -208,6 +210,47 @@ void folder_item_append(FolderItem *parent, FolderItem *item)
 	item->node = g_node_append_data(parent->node, item);
 }
 
+FolderItem *folder_item_copy(FolderItem *item)
+{
+	FolderItem *new_item;
+
+	new_item = g_new0(FolderItem, 1);
+
+	new_item->stype = item->stype;
+	new_item->name = g_strdup(item->name);
+	new_item->path = g_strdup(item->path);
+	new_item->mtime = item->mtime;
+	new_item->new = item->new;
+	new_item->unread = item->unread;
+	new_item->total = item->total;
+	new_item->unmarked_num = item->unmarked_num;
+	new_item->last_num = item->last_num;
+	new_item->no_sub = item->no_sub;
+	new_item->no_select = item->no_select;
+	new_item->collapsed = item->collapsed;
+	new_item->threaded = item->threaded;
+	new_item->opened = item->opened;
+	new_item->updated = item->updated;
+	new_item->cache_dirty = item->cache_dirty;
+	new_item->mark_dirty = item->mark_dirty;
+	new_item->node = item->node;
+	new_item->parent = item->parent;
+	new_item->folder = item->folder;
+	new_item->account = item->account;
+	new_item->ac_apply_sub = item->ac_apply_sub;
+	new_item->auto_to = g_strdup(item->auto_to);
+	new_item->use_auto_to_on_reply = item->use_auto_to_on_reply;
+	new_item->auto_cc = g_strdup(item->auto_cc);
+	new_item->auto_bcc = g_strdup(item->auto_bcc);
+	new_item->auto_replyto = g_strdup(item->auto_replyto);
+	new_item->mark_queue = item->mark_queue;
+	new_item->last_selected = item->last_selected;
+	new_item->qsearch_cond_type = item->qsearch_cond_type;
+	new_item->data = item->data;
+
+	return new_item;
+}
+
 static gboolean folder_item_remove_func(GNode *node, gpointer data)
 {
 	FolderItem *item = FOLDER_ITEM(node->data);
@@ -293,8 +336,10 @@ gint folder_item_compare(FolderItem *item_a, FolderItem *item_b)
 
 	/* if both a and b are special folders, sort them according to
 	 * their types (which is in-order). Note that this assumes that
-	 * there are no multiple folders of a special type. */
-	if (item_a->stype != F_NORMAL && item_b->stype != F_NORMAL)
+	 * there are no multiple folders of a special type. As a special
+	 * case, two virtual folders are compared like normal ones. */
+	if (item_a->stype != F_NORMAL && item_b->stype != F_NORMAL &&
+	    (item_a->stype != F_VIRTUAL || item_b->stype != F_VIRTUAL))
 		return item_a->stype - item_b->stype;
 
 	 /* if b is normal folder, and a is not, b is smaller (ends up
@@ -428,6 +473,8 @@ void folder_write_list(void)
 
 	if (prefs_file_close(pfile) < 0)
 		g_warning("failed to write folder list.\n");
+
+	g_signal_emit_by_name(syl_app_get(), "folderlist-updated");
 }
 
 struct TotalMsgStatus
@@ -727,6 +774,24 @@ FolderItem *folder_find_item_from_identifier(const gchar *identifier)
 	return d[1];
 }
 
+FolderItem *folder_find_item_and_num_from_id(const gchar *identifier, gint *num)
+{
+	gchar *id;
+	gchar *msg;
+	FolderItem *item;
+
+	g_return_val_if_fail(identifier != NULL, NULL);
+
+	id = g_path_get_dirname(identifier);
+	msg = g_path_get_basename(identifier);
+	item = folder_find_item_from_identifier(id);
+	*num = to_number(msg);
+	g_free(msg);
+	g_free(id);
+
+	return item;
+}
+
 Folder *folder_get_default_folder(void)
 {
 	return folder_list ? FOLDER(folder_list->data) : NULL;
@@ -1250,6 +1315,7 @@ static gboolean folder_build_tree(GNode *node, gpointer data)
 		 threaded = TRUE, ac_apply_sub = FALSE;
 	FolderSortKey sort_key = SORT_BY_NONE;
 	FolderSortType sort_type = SORT_ASCENDING;
+	gboolean qsearch_cond_type = 0;
 	gint new = 0, unread = 0, total = 0;
 	time_t mtime = 0;
 	gboolean use_auto_to_on_reply = FALSE;
@@ -1345,6 +1411,21 @@ static gboolean folder_build_tree(GNode *node, gpointer data)
 				sort_type = SORT_ASCENDING;
 			else
 				sort_type = SORT_DESCENDING;
+		} else if (!strcmp(attr->name, "qsearch_cond")) {
+			if (!strcmp(attr->value, "all"))
+				qsearch_cond_type = 0;
+			else if (!strcmp(attr->value, "unread"))
+				qsearch_cond_type = 1;
+			else if (!strcmp(attr->value, "mark"))
+				qsearch_cond_type = 2;
+			else if (!strcmp(attr->value, "clabel"))
+				qsearch_cond_type = 3;
+			else if (!strcmp(attr->value, "mime"))
+				qsearch_cond_type = 4;
+			else if (!strcmp(attr->value, "w1day"))
+				qsearch_cond_type = 5;
+			else if (!strcmp(attr->value, "last5"))
+				qsearch_cond_type = 6;
 		} else if (!strcmp(attr->name, "account_id")) {
 			account = account_find_from_id(atoi(attr->value));
 			if (!account) g_warning("account_id: %s not found\n",
@@ -1404,6 +1485,7 @@ static gboolean folder_build_tree(GNode *node, gpointer data)
 	item->auto_replyto = auto_replyto;
 	item->trim_summary_subject = trim_summary_subject;
 	item->trim_compose_subject = trim_compose_subject;
+	item->qsearch_cond_type = qsearch_cond_type;
 	node->data = item;
 	xml_free_node(xmlnode);
 
@@ -1520,6 +1602,8 @@ static void folder_write_list_recursive(GNode *node, gpointer data)
 					"thread-date",
 					"from", "subject", "score", "label",
 					"mark", "unread", "mime", "to"};
+	static gchar *qsearch_cond_str[] = {"all", "unread", "mark", "clabel",
+					    "mime", "w1day", "last5"};
 
 	g_return_if_fail(node != NULL);
 	g_return_if_fail(fp != NULL);
@@ -1577,6 +1661,11 @@ static void folder_write_list_recursive(GNode *node, gpointer data)
 			else
 				fprintf(fp, " sort_type=\"descending\"");
 		}
+		if (item->qsearch_cond_type > 0 &&
+		    item->qsearch_cond_type < 7) {
+			fprintf(fp, " qsearch_cond=\"%s\"",
+				qsearch_cond_str[item->qsearch_cond_type]);
+		}
 
 		fprintf(fp,
 			" mtime=\"%lu\" new=\"%d\" unread=\"%d\" total=\"%d\"",
