@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2008 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,10 +44,10 @@
 #include "account.h"
 #include "procmsg.h"
 #include "socket.h"
-#include "ssl.h"
 #include "pop.h"
 #include "recv.h"
 #include "mbox.h"
+#include "imap.h"
 #include "utils.h"
 #include "gtkutils.h"
 #include "statusbar.h"
@@ -62,13 +62,12 @@
 static GList *inc_dialog_list = NULL;
 
 static guint inc_lock_count = 0;
+static gboolean block_notify = FALSE;
 
 static GdkPixbuf *current_pixbuf;
 static GdkPixbuf *error_pixbuf;
 static GdkPixbuf *ok_pixbuf;
 
-#define MSGBUFSIZE	8192
-
 static void inc_finished		(MainWindow		*mainwin,
 					 gint			 new_messages);
 
@@ -154,16 +153,17 @@ static void inc_finished(MainWindow *mainwin, gint new_messages)
 	if (prefs_common.scan_all_after_inc)
 		new_messages += folderview_check_new(NULL);
 
-	if (new_messages > 0) {
+	if (new_messages > 0 && !block_notify) {
 		gchar buf[1024];
 
 		g_snprintf(buf, sizeof(buf), _("Sylpheed: %d new messages"),
 			   new_messages);
 		trayicon_set_tooltip(buf);
-	} else {
-		trayicon_set_tooltip(NULL);
+		trayicon_set_notify(TRUE);
 	}
 
+	inc_block_notify(FALSE);
+
 	if (new_messages <= 0 && !prefs_common.scan_all_after_inc) return;
 
 	if (prefs_common.open_inbox_on_inc) {
@@ -200,6 +200,7 @@ void inc_mail(MainWindow *mainwin)
 	gint new_msgs = 0;
 
 	if (inc_lock_count) return;
+	if (inc_is_active()) return;
 
 	if (!main_window_toggle_online_if_offline(mainwin))
 		return;
@@ -236,24 +237,114 @@ void inc_mail(MainWindow *mainwin)
 static gint inc_remote_account_mail(MainWindow *mainwin, PrefsAccount *account)
 {
 	FolderItem *item = mainwin->summaryview->folder_item;
-	gint new_msgs;
+	gint new_msgs = 0;
+	gboolean update_summary = FALSE;
 
 	g_return_val_if_fail(account->folder != NULL, 0);
 
+	if (account->protocol == A_IMAP4 &&
+	    account->imap_filter_inbox_on_recv) {
+		FolderItem *inbox = FOLDER(account->folder)->inbox;
+		GSList *mlist, *cur;
+		FilterInfo *fltinfo;
+		gint n_filtered = 0;
+
+		debug_print("inc_remote_account_mail(): filtering IMAP4 INBOX\n");
+		mlist = folder_item_get_uncached_msg_list(inbox);
+		debug_print("inc_remote_account_mail(): uncached messages: %d\n", g_slist_length(mlist));
+
+		for (cur = mlist; cur != NULL; cur = cur->next) {
+			MsgInfo *msginfo = (MsgInfo *)cur->data;
+
+			fltinfo = filter_info_new();
+			fltinfo->account = account;
+			fltinfo->flags = msginfo->flags;
+
+			if (prefs_common.enable_junk &&
+			    prefs_common.filter_junk_on_recv &&
+			    prefs_common.filter_junk_before) {
+				filter_apply_msginfo
+					(prefs_common.manual_junk_fltlist,
+					 msginfo, fltinfo);
+			}
+
+			if (!fltinfo->drop_done) {
+				filter_apply_msginfo(prefs_common.fltlist,
+						     msginfo, fltinfo);
+			}
+
+			if (!fltinfo->drop_done &&
+			    prefs_common.enable_junk &&
+			    prefs_common.filter_junk_on_recv &&
+			    !prefs_common.filter_junk_before) {
+				filter_apply_msginfo
+					(prefs_common.manual_junk_fltlist,
+					 msginfo, fltinfo);
+			}
+
+			if (msginfo->flags.perm_flags !=
+			    fltinfo->flags.perm_flags) {
+				msginfo->flags = fltinfo->flags;
+				inbox->mark_dirty = TRUE;
+				if (fltinfo->actions[FLT_ACTION_MARK])
+					imap_msg_set_perm_flags
+						(msginfo, MSG_MARKED);
+				if (fltinfo->actions[FLT_ACTION_MARK_READ])
+					imap_msg_unset_perm_flags
+						(msginfo, MSG_NEW|MSG_UNREAD);
+			}
+
+			if (fltinfo->actions[FLT_ACTION_MOVE] &&
+			    fltinfo->move_dest) {
+				folder_item_move_msg
+					(fltinfo->move_dest, msginfo);
+				if (account->imap_check_inbox_only ||
+				    fltinfo->move_dest->folder !=
+				    inbox->folder) {
+					if (MSG_IS_NEW(fltinfo->flags) ||
+					    MSG_IS_UNREAD(fltinfo->flags))
+						++new_msgs;
+				}
+			} else if (fltinfo->actions[FLT_ACTION_DELETE])
+				folder_item_remove_msg(inbox, msginfo);
+			else if (MSG_IS_NEW(msginfo->flags) ||
+				 MSG_IS_UNREAD(msginfo->flags))
+				++new_msgs;
+
+			if (fltinfo->drop_done)
+				++n_filtered;
+
+			filter_info_free(fltinfo);
+		}
+
+		procmsg_msg_list_free(mlist);
+
+		debug_print("inc_remote_account_mail(): INBOX: %d new, %d filtered\n",
+			    new_msgs, n_filtered);
+
+		if (!prefs_common.scan_all_after_inc && item != NULL &&
+		    inbox == item)
+			update_summary = TRUE;
+	}
+
 	if (account->protocol == A_IMAP4 && account->imap_check_inbox_only) {
 		FolderItem *inbox = FOLDER(account->folder)->inbox;
 
-		new_msgs = folderview_check_new_item(inbox);
+		new_msgs += folderview_check_new_item(inbox);
 		if (!prefs_common.scan_all_after_inc && item != NULL &&
 		    inbox == item)
-			folderview_update_item(item, TRUE);
+			update_summary = TRUE;
 	} else {
-		new_msgs = folderview_check_new(FOLDER(account->folder));
+		new_msgs += folderview_check_new(FOLDER(account->folder));
 		if (!prefs_common.scan_all_after_inc && item != NULL &&
 		    FOLDER(account->folder) == item->folder)
-			folderview_update_item(item, TRUE);
+			update_summary = TRUE;
 	}
 
+	if (update_summary)
+		folderview_update_item(item, TRUE);
+	folderview_update_all_updated(FALSE);
+
 	return new_msgs;
 }
 
@@ -284,6 +375,7 @@ gint inc_account_mail(MainWindow *mainwin, PrefsAccount *account)
 	gint new_msgs;
 
 	if (inc_lock_count) return 0;
+	if (inc_is_active()) return 0;
 
 	if (!main_window_toggle_online_if_offline(mainwin))
 		return 0;
@@ -308,6 +400,7 @@ void inc_all_account_mail(MainWindow *mainwin, gboolean autocheck)
 	gint new_msgs = 0;
 
 	if (inc_lock_count) return;
+	if (inc_is_active()) return;
 
 	if (!main_window_toggle_online_if_offline(mainwin))
 		return;
@@ -359,6 +452,45 @@ void inc_all_account_mail(MainWindow *mainwin, gboolean autocheck)
 	inc_autocheck_timer_set();
 }
 
+gint inc_pop_before_smtp(PrefsAccount *account)
+{
+	MainWindow *mainwin;
+	IncProgressDialog *inc_dialog;
+	IncSession *session;
+
+	if (inc_lock_count) return -1;
+
+	mainwin = main_window_get();
+
+	if (!main_window_toggle_online_if_offline(mainwin))
+		return -1;
+
+	inc_autocheck_timer_remove();
+	main_window_lock(mainwin);
+
+	session = inc_session_new(account);
+	if (!session) return -1;
+	POP3_SESSION(session->session)->auth_only = TRUE;
+
+	inc_dialog = inc_progress_dialog_create(FALSE);
+	gtk_window_set_title(GTK_WINDOW(inc_dialog->dialog->window),
+			     _("Authenticating with POP3"));
+	inc_dialog->queue_list = g_list_append(inc_dialog->queue_list, session);
+	inc_dialog->mainwin = mainwin;
+	inc_progress_dialog_set_list(inc_dialog);
+	inc_dialog->show_dialog = TRUE;
+
+	main_window_set_toolbar_sensitive(mainwin);
+	main_window_set_menu_sensitive(mainwin);
+
+	inc_start(inc_dialog);
+
+	main_window_unlock(mainwin);
+	inc_autocheck_timer_set();
+
+	return 0;
+}
+
 static IncProgressDialog *inc_progress_dialog_create(gboolean autocheck)
 {
 	IncProgressDialog *dialog;
@@ -411,7 +543,7 @@ static void inc_progress_dialog_set_list(IncProgressDialog *inc_dialog)
 		session->data = inc_dialog;
 		progress_dialog_append(inc_dialog->dialog, NULL,
 				       pop3_session->ac_prefs->account_name,
-				       _("Standby"), NULL);
+				       _("Standby"), "", NULL);
 	}
 }
 
@@ -460,9 +592,14 @@ static IncSession *inc_session_new(PrefsAccount *account)
 	session_set_recv_data_notify(session->session,
 				     inc_recv_data_finished, session);
 
+	session->inc_state = INC_SUCCESS;
+
 	session->folder_table = g_hash_table_new(NULL, NULL);
 	session->tmp_folder_table = g_hash_table_new(NULL, NULL);
 
+	session->cur_total_bytes = 0;
+	session->new_msgs = 0;
+
 	return session;
 }
 
@@ -492,15 +629,13 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 		GList *next = qlist->next;
 
 		session = qlist->data;
+		if (session->inc_state == INC_CANCEL) {
+			qlist = next;
+			continue;
+		}
+
 		pop3_session = POP3_SESSION(session->session); 
-		pop3_session->user = g_strdup(pop3_session->ac_prefs->userid);
-		if (pop3_session->ac_prefs->passwd)
-			pop3_session->pass =
-				g_strdup(pop3_session->ac_prefs->passwd);
-		else if (pop3_session->ac_prefs->tmp_pass)
-			pop3_session->pass =
-				g_strdup(pop3_session->ac_prefs->tmp_pass);
-		else {
+		if (!pop3_session->pass) {
 			gchar *pass;
 
 			if (inc_dialog->show_dialog)
@@ -527,20 +662,25 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 		qlist = next;
 	}
 
-#define SET_PIXMAP_AND_TEXT(pixbuf, str)				\
+#define SET_PIXMAP_AND_TEXT(pixbuf, status, progress)			\
 {									\
 	progress_dialog_set_row_pixbuf(inc_dialog->dialog,		\
 				       inc_dialog->cur_row, pixbuf);	\
 	progress_dialog_set_row_status(inc_dialog->dialog,		\
-				       inc_dialog->cur_row, str);	\
+				       inc_dialog->cur_row, status);	\
+	if (progress)							\
+		progress_dialog_set_row_progress(inc_dialog->dialog,	\
+						 inc_dialog->cur_row,	\
+						 progress);		\
 }
 
 	for (; inc_dialog->queue_list != NULL; inc_dialog->cur_row++) {
 		session = inc_dialog->queue_list->data;
 		pop3_session = POP3_SESSION(session->session);
 
-		if (pop3_session->pass == NULL) {
-			SET_PIXMAP_AND_TEXT(ok_pixbuf, _("Cancelled"));
+		if (session->inc_state == INC_CANCEL ||
+		    pop3_session->pass == NULL) {
+			SET_PIXMAP_AND_TEXT(ok_pixbuf, _("Cancelled"), NULL);
 			inc_session_destroy(session);
 			inc_dialog->queue_list =
 				g_list_remove(inc_dialog->queue_list, session);
@@ -551,7 +691,7 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 		progress_dialog_scroll_to_row(inc_dialog->dialog,
 					      inc_dialog->cur_row);
 
-		SET_PIXMAP_AND_TEXT(current_pixbuf, _("Retrieving"));
+		SET_PIXMAP_AND_TEXT(current_pixbuf, _("Retrieving"), NULL);
 
 		/* begin POP3 session */
 		inc_state = inc_pop3_session_do(session);
@@ -560,42 +700,43 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 		case INC_SUCCESS:
 			if (pop3_session->cur_total_num > 0)
 				msg = g_strdup_printf
-					(_("Done (%d message(s) (%s) received)"),
+					(_("%d message(s) (%s) received"),
 					 pop3_session->cur_total_num,
 					 to_human_readable(pop3_session->cur_total_recv_bytes));
 			else
-				msg = g_strdup_printf(_("Done (no new messages)"));
-			SET_PIXMAP_AND_TEXT(ok_pixbuf, msg);
+				msg = g_strdup_printf(_("no new messages"));
+			SET_PIXMAP_AND_TEXT(ok_pixbuf, _("Done"), msg);
 			g_free(msg);
 			break;
 		case INC_CONNECT_ERROR:
 			SET_PIXMAP_AND_TEXT(error_pixbuf,
-					    _("Connection failed"));
+					    _("Connection failed"), NULL);
 			break;
 		case INC_AUTH_FAILED:
-			SET_PIXMAP_AND_TEXT(error_pixbuf, _("Auth failed"));
+			SET_PIXMAP_AND_TEXT(error_pixbuf, _("Auth failed"),
+					    NULL);
 			break;
 		case INC_LOCKED:
-			SET_PIXMAP_AND_TEXT(error_pixbuf, _("Locked"));
+			SET_PIXMAP_AND_TEXT(error_pixbuf, _("Locked"), NULL);
 			break;
 		case INC_ERROR:
 		case INC_NO_SPACE:
 		case INC_IO_ERROR:
 		case INC_SOCKET_ERROR:
 		case INC_EOF:
-			SET_PIXMAP_AND_TEXT(error_pixbuf, _("Error"));
+			SET_PIXMAP_AND_TEXT(error_pixbuf, _("Error"), NULL);
 			break;
 		case INC_TIMEOUT:
-			SET_PIXMAP_AND_TEXT(error_pixbuf, _("Timeout"));
+			SET_PIXMAP_AND_TEXT(error_pixbuf, _("Timeout"), NULL);
 			break;
 		case INC_CANCEL:
-			SET_PIXMAP_AND_TEXT(ok_pixbuf, _("Cancelled"));
+			SET_PIXMAP_AND_TEXT(ok_pixbuf, _("Cancelled"), NULL);
 			break;
 		default:
 			break;
 		}
 
-		new_msgs += pop3_session->cur_total_num;
+		new_msgs += session->new_msgs;
 
 		if (!prefs_common.scan_all_after_inc) {
 			folder_item_scan_foreach(session->folder_table);
@@ -680,33 +821,22 @@ static IncState inc_pop3_session_do(IncSession *session)
 {
 	Pop3Session *pop3_session = POP3_SESSION(session->session);
 	IncProgressDialog *inc_dialog = (IncProgressDialog *)session->data;
-	gchar *server;
-	gushort port;
 	gchar *buf;
 
 	debug_print(_("getting new messages of account %s...\n"),
 		    pop3_session->ac_prefs->account_name);
 
-	buf = g_strdup_printf(_("%s: Retrieving new messages"),
-			      pop3_session->ac_prefs->recv_server);
+	if (pop3_session->auth_only)
+		buf = g_strdup_printf(_("%s: Authenticating with POP3"),
+				      pop3_session->ac_prefs->recv_server);
+	else
+		buf = g_strdup_printf(_("%s: Retrieving new messages"),
+				      pop3_session->ac_prefs->recv_server);
 	gtk_window_set_title(GTK_WINDOW(inc_dialog->dialog->window), buf);
 	g_free(buf);
 
-	server = pop3_session->ac_prefs->recv_server;
-#if USE_SSL
-	port = pop3_session->ac_prefs->set_popport ?
-		pop3_session->ac_prefs->popport :
-		pop3_session->ac_prefs->ssl_pop == SSL_TUNNEL ? 995 : 110;
-	SESSION(pop3_session)->ssl_type = pop3_session->ac_prefs->ssl_pop;
-	if (pop3_session->ac_prefs->ssl_pop != SSL_NONE)
-		SESSION(pop3_session)->nonblocking =
-			pop3_session->ac_prefs->use_nonblocking_ssl;
-#else
-	port = pop3_session->ac_prefs->set_popport ?
-		pop3_session->ac_prefs->popport : 110;
-#endif
-
-	buf = g_strdup_printf(_("Connecting to POP3 server: %s..."), server);
+	buf = g_strdup_printf(_("Connecting to POP3 server: %s..."),
+			      pop3_session->ac_prefs->recv_server);
 	log_message("%s\n", buf);
 	progress_dialog_set_label(inc_dialog->dialog, buf);
 	g_free(buf);
@@ -716,9 +846,12 @@ static IncState inc_pop3_session_do(IncSession *session)
 
 	GTK_EVENTS_FLUSH();
 
-	if (session_connect(SESSION(pop3_session), server, port) < 0) {
+	if (session_connect(SESSION(pop3_session),
+			    SESSION(pop3_session)->server,
+			    SESSION(pop3_session)->port) < 0) {
 		log_warning(_("Can't connect to POP3 server: %s:%d\n"),
-			    server, port);
+			    SESSION(pop3_session)->server,
+			    SESSION(pop3_session)->port);
 		session->inc_state = INC_CONNECT_ERROR;
 		statusbar_pop_all();
 		return INC_CONNECT_ERROR;
@@ -823,7 +956,7 @@ static void inc_progress_dialog_set_label(IncProgressDialog *inc_dialog,
 #if 0
 		if (session->msg[session->cur_msg].recv_time <
 			session->current_time) {
-			gchar buf[MSGBUFSIZE];
+			gchar buf[BUFFSIZE];
 			g_snprintf(buf, sizeof(buf), _("Deleting message %d"),
 				   session->cur_msg);
 			progress_dialog_set_label(dialog, buf);
@@ -841,7 +974,7 @@ static void inc_progress_dialog_set_label(IncProgressDialog *inc_dialog,
 static void inc_progress_dialog_set_progress(IncProgressDialog *inc_dialog,
 					     IncSession *inc_session)
 {
-	gchar buf[MSGBUFSIZE];
+	gchar buf[BUFFSIZE];
 	Pop3Session *pop3_session = POP3_SESSION(inc_session->session);
 	gchar *total_size_str;
 	gint64 cur_total;
@@ -877,12 +1010,12 @@ static void inc_progress_dialog_set_progress(IncProgressDialog *inc_dialog,
 
 	if (pop3_session->cur_total_num > 0) {
 		g_snprintf(buf, sizeof(buf),
-			   _("Retrieving (%d message(s) (%s) received)"),
+			   _("%d message(s) (%s) received"),
 			   pop3_session->cur_total_num,
 			   to_human_readable
 			   (pop3_session->cur_total_recv_bytes));
-		progress_dialog_set_row_status(inc_dialog->dialog,
-					       inc_dialog->cur_row, buf);
+		progress_dialog_set_row_progress(inc_dialog->dialog,
+						 inc_dialog->cur_row, buf);
 	}
 }
 
@@ -1038,6 +1171,7 @@ static gint inc_drop_message(Pop3Session *session, const gchar *file)
 	FilterInfo *fltinfo;
 	IncSession *inc_session = (IncSession *)(SESSION(session)->data);
 	gint val;
+	gboolean is_junk = FALSE;
 
 	g_return_val_if_fail(inc_session != NULL, DROP_ERROR);
 
@@ -1058,8 +1192,19 @@ static gint inc_drop_message(Pop3Session *session, const gchar *file)
 
 	if (prefs_common.enable_junk &&
 	    prefs_common.filter_junk_on_recv &&
-	    prefs_common.filter_junk_before)
+	    prefs_common.filter_junk_before) {
 		filter_apply(prefs_common.junk_fltlist, file, fltinfo);
+		if (fltinfo->drop_done)
+			is_junk = TRUE;
+		else if (fltinfo->error == FLT_ERROR_EXEC_FAILED) {
+			alertpanel_error
+				(_("Execution of the junk filter command failed.\n"
+				   "Please check the junk mail control setting."));
+			filter_info_free(fltinfo);
+			inc_session->inc_state = INC_ERROR;
+			return DROP_ERROR;
+		}
+	}
 
 	if (!fltinfo->drop_done && session->ac_prefs->filter_on_recv)
 		filter_apply(prefs_common.fltlist, file, fltinfo);
@@ -1067,8 +1212,19 @@ static gint inc_drop_message(Pop3Session *session, const gchar *file)
 	if (!fltinfo->drop_done) {
 		if (prefs_common.enable_junk &&
 		    prefs_common.filter_junk_on_recv &&
-		    !prefs_common.filter_junk_before)
+		    !prefs_common.filter_junk_before) {
 			filter_apply(prefs_common.junk_fltlist, file, fltinfo);
+			if (fltinfo->drop_done)
+				is_junk = TRUE;
+			else if (fltinfo->error == FLT_ERROR_EXEC_FAILED) {
+				alertpanel_error
+					(_("Execution of the junk filter command failed.\n"
+					   "Please check the junk mail control setting."));
+				filter_info_free(fltinfo);
+				inc_session->inc_state = INC_ERROR;
+				return DROP_ERROR;
+			}
+		}
 	}
 
 	if (!fltinfo->drop_done) {
@@ -1096,8 +1252,11 @@ static gint inc_drop_message(Pop3Session *session, const gchar *file)
 		val = DROP_DONT_RECEIVE;
 	else if (fltinfo->actions[FLT_ACTION_DELETE] == TRUE)
 		val = DROP_DELETE;
-	else
+	else {
 		val = DROP_OK;
+		if (!is_junk && fltinfo->actions[FLT_ACTION_MARK_READ] == FALSE)
+			inc_session->new_msgs++;
+	}
 
 	filter_info_free(fltinfo);
 
@@ -1187,14 +1346,15 @@ static void inc_put_error(IncState istate, const gchar *msg)
 			log_warning("%s\n", log_msg);
 	}
 	if (err_msg) {
-		alertpanel_error(err_msg);
+		alertpanel_error("%s", err_msg);
 		g_free(err_msg);
 	}
 }
 
-static void inc_cancel(IncProgressDialog *dialog)
+static void inc_cancel(IncProgressDialog *dialog, gboolean cancel_all)
 {
 	IncSession *session;
+	GList *list;
 
 	g_return_if_fail(dialog != NULL);
 
@@ -1203,16 +1363,39 @@ static void inc_cancel(IncProgressDialog *dialog)
 		return;
 	}
 
-	session = dialog->queue_list->data;
-
-	session->inc_state = INC_CANCEL;
+	for (list = dialog->queue_list; list != NULL; list = list->next) {
+		session = list->data;
+		session->inc_state = INC_CANCEL;
+		session_disconnect(session->session);
+		if (!cancel_all)
+			break;
+	}
 
 	log_message(_("Incorporation cancelled\n"));
 }
 
 gboolean inc_is_active(void)
 {
-	return (inc_dialog_list != NULL);
+	GList *cur;
+
+	if (inc_dialog_list == NULL)
+		return FALSE;
+
+	for (cur = inc_dialog_list; cur != NULL; cur = cur->next) {
+		IncProgressDialog *dialog = cur->data;
+		if (dialog->queue_list)
+			return TRUE;
+	}
+
+	return FALSE;
+}
+
+void inc_block_notify(gboolean block)
+{
+	if (!block)
+		block_notify = FALSE;
+	else if (inc_is_active())
+		block_notify = TRUE;
 }
 
 void inc_cancel_all(void)
@@ -1220,12 +1403,12 @@ void inc_cancel_all(void)
 	GList *cur;
 
 	for (cur = inc_dialog_list; cur != NULL; cur = cur->next)
-		inc_cancel((IncProgressDialog *)cur->data);
+		inc_cancel((IncProgressDialog *)cur->data, TRUE);
 }
 
 static void inc_cancel_cb(GtkWidget *widget, gpointer data)
 {
-	inc_cancel((IncProgressDialog *)data);
+	inc_cancel((IncProgressDialog *)data, FALSE);
 }
 
 static gint inc_dialog_delete_cb(GtkWidget *widget, GdkEventAny *event,
@@ -1293,26 +1476,25 @@ static gint get_spool(FolderItem *dest, const gchar *mbox)
 	debug_print(_("Getting new messages from %s into %s...\n"),
 		    mbox, dest->path);
 
-	if (prefs_common.filter_on_inc)
-		folder_table = g_hash_table_new(NULL, NULL);
-	msgs = proc_mbox(dest, tmp_mbox, folder_table);
+	folder_table = g_hash_table_new(NULL, NULL);
+
+	msgs = proc_mbox_full(dest, tmp_mbox, folder_table,
+			      prefs_common.filter_on_inc,
+			      prefs_common.enable_junk &&
+			      prefs_common.filter_junk_on_recv);
 
 	g_unlink(tmp_mbox);
 	if (msgs >= 0) empty_mbox(mbox);
 	unlock_mbox(mbox, lockfd, LOCK_FLOCK);
 
-	if (folder_table) {
-		if (!prefs_common.scan_all_after_inc) {
-			folder_item_scan_foreach(folder_table);
-			folderview_update_item_foreach
-				(folder_table, !prefs_common.open_inbox_on_inc);
-		}
-		g_hash_table_destroy(folder_table);
-	} else if (!prefs_common.scan_all_after_inc) {
-		folder_item_scan(dest);
-		folderview_update_item(dest, TRUE);
+	if (!prefs_common.scan_all_after_inc) {
+		folder_item_scan_foreach(folder_table);
+		folderview_update_item_foreach
+			(folder_table, !prefs_common.open_inbox_on_inc);
 	}
 
+	g_hash_table_destroy(folder_table);
+
 	return msgs;
 }
 
