@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2008 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,24 +31,32 @@
 #include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
-#include <sys/wait.h>
+#if HAVE_SYS_WAIT_H
+#  include <sys/wait.h>
+#endif
 
 #include "send_message.h"
 #include "session.h"
 #include "ssl.h"
 #include "smtp.h"
 #include "news.h"
+#include "imap.h"
 #include "prefs_common.h"
 #include "prefs_account.h"
 #include "procheader.h"
 #include "account.h"
+#include "folder.h"
+#include "procmsg.h"
+#include "filter.h"
 #include "progressdialog.h"
 #include "statusbar.h"
-#include "inputdialog.h"
 #include "alertpanel.h"
 #include "manage_window.h"
 #include "socket.h"
 #include "utils.h"
+#include "inc.h"
+#include "mainwindow.h"
+#include "summaryview.h"
 
 #define SMTP_PORT	25
 #if USE_SSL
@@ -99,7 +107,7 @@ gint send_message(const gchar *file, PrefsAccount *ac_prefs, GSList *to_list)
 	g_return_val_if_fail(ac_prefs != NULL, -1);
 	g_return_val_if_fail(to_list != NULL, -1);
 
-	if ((fp = fopen(file, "rb")) == NULL) {
+	if ((fp = g_fopen(file, "rb")) == NULL) {
 		FILE_OP_ERROR(file, "fopen");
 		return -1;
 	}
@@ -121,7 +129,9 @@ enum
 	Q_SENDER     = 0,
 	Q_SMTPSERVER = 1,
 	Q_RECIPIENTS = 2,
-	Q_ACCOUNT_ID = 3
+	Q_ACCOUNT_ID = 3,
+	Q_REPLY_TARGET = 4,
+	Q_FORWARD_TARGETS = 5
 };
 
 QueueInfo *send_get_queue_info(const gchar *file)
@@ -130,6 +140,8 @@ QueueInfo *send_get_queue_info(const gchar *file)
 				       {"SSV:", NULL, FALSE},
 				       {"R:",   NULL, FALSE},
 				       {"AID:", NULL, FALSE},
+				       {"REP:", NULL, FALSE},
+				       {"FWD:", NULL, FALSE},
 				       {NULL,   NULL, FALSE}};
 	FILE *fp;
 	gchar buf[BUFFSIZE];
@@ -138,7 +150,7 @@ QueueInfo *send_get_queue_info(const gchar *file)
 
 	g_return_val_if_fail(file != NULL, NULL);
 
-	if ((fp = fopen(file, "rb")) == NULL) {
+	if ((fp = g_fopen(file, "rb")) == NULL) {
 		FILE_OP_ERROR(file, "fopen");
 		return NULL;
 	}
@@ -167,6 +179,12 @@ QueueInfo *send_get_queue_info(const gchar *file)
 		case Q_ACCOUNT_ID:
 			qinfo->ac = account_find_from_id(atoi(p));
 			break;
+		case Q_REPLY_TARGET:
+			qinfo->reply_target = g_strdup(p);
+			break;
+		case Q_FORWARD_TARGETS:
+			qinfo->forward_targets = g_strdup(p);
+			break;
 		default:
 			break;
 		}
@@ -204,7 +222,7 @@ gint send_get_queue_contents(QueueInfo *qinfo, const gchar *dest)
 	g_return_val_if_fail(qinfo->fp != NULL, -1);
 	g_return_val_if_fail(dest != NULL, -1);
 
-	if ((fp = fopen(dest, "wb")) == NULL) {
+	if ((fp = g_fopen(dest, "wb")) == NULL) {
 		FILE_OP_ERROR(dest, "fopen");
 		return -1;
 	}
@@ -216,7 +234,7 @@ gint send_get_queue_contents(QueueInfo *qinfo, const gchar *dest)
 
 	if (fclose(fp) < 0) {
 		FILE_OP_ERROR(dest, "fclose");
-		unlink(dest);
+		g_unlink(dest);
 		return -1;
 	}
 
@@ -233,6 +251,8 @@ void send_queue_info_free(QueueInfo *qinfo)
 	g_slist_free(qinfo->to_list);
 	g_free(qinfo->from);
 	g_free(qinfo->server);
+	g_free(qinfo->reply_target);
+	g_free(qinfo->forward_targets);
 	if (qinfo->fp)
 		fclose(qinfo->fp);
 	g_free(qinfo);
@@ -302,6 +322,228 @@ gint send_message_queue(QueueInfo *qinfo)
 	return val;
 }
 
+gint send_message_queue_all(FolderItem *queue, gboolean save_msgs,
+			    gboolean filter_msgs)
+{
+	gint ret = 0;
+	GSList *mlist = NULL;
+	GSList *cur;
+
+	if (!queue)
+		queue = folder_get_default_queue();
+	g_return_val_if_fail(queue != NULL, -1);
+
+	mlist = folder_item_get_msg_list(queue, FALSE);
+	mlist = procmsg_sort_msg_list(mlist, SORT_BY_NUMBER, SORT_ASCENDING);
+
+	for (cur = mlist; cur != NULL; cur = cur->next) {
+		gchar *file;
+		MsgInfo *msginfo = (MsgInfo *)cur->data;
+		QueueInfo *qinfo;
+		gchar tmp[MAXPATHLEN + 1];
+
+		file = procmsg_get_message_file(msginfo);
+		if (!file)
+			continue;
+
+		qinfo = send_get_queue_info(file);
+		if (!qinfo || send_message_queue(qinfo) < 0) {
+			g_warning("Sending queued message %d failed.\n",
+				  msginfo->msgnum);
+			send_queue_info_free(qinfo);
+			g_free(file);
+			continue;
+		}
+
+		if (qinfo->reply_target)
+			send_message_set_reply_flag(qinfo->reply_target,
+						    msginfo->inreplyto);
+		else if (qinfo->forward_targets)
+			send_message_set_forward_flags(qinfo->forward_targets);
+
+		g_snprintf(tmp, sizeof(tmp), "%s%ctmpmsg.out.%08x",
+			   get_rc_dir(), G_DIR_SEPARATOR, g_random_int());
+
+		if (send_get_queue_contents(qinfo, tmp) == 0) {
+			if (save_msgs) {
+				FolderItem *outbox;
+				gboolean drop_done = FALSE;
+
+				if (filter_msgs) {
+					FilterInfo *fltinfo;
+
+					fltinfo = filter_info_new();
+					fltinfo->account = qinfo->ac;
+					fltinfo->flags.perm_flags = 0;
+					fltinfo->flags.tmp_flags = MSG_RECEIVED;
+
+					filter_apply(prefs_common.fltlist, tmp,
+						     fltinfo);
+
+					drop_done = fltinfo->drop_done;
+					filter_info_free(fltinfo);
+				}
+
+				if (!drop_done) {
+					outbox = account_get_special_folder
+						(qinfo->ac, F_OUTBOX);
+					procmsg_save_to_outbox(outbox, tmp);
+				}
+			}
+			g_unlink(tmp);
+		}
+
+		send_queue_info_free(qinfo);
+		g_free(file);
+
+		folder_item_remove_msg(queue, msginfo);
+		ret++;
+	}
+
+	procmsg_msg_list_free(mlist);
+
+	procmsg_clear_cache(queue);
+	queue->cache_dirty = FALSE;
+	queue->mtime = 0;
+
+	return ret;
+}
+
+gint send_message_set_reply_flag(const gchar *target, const gchar *msgid)
+{
+	FolderItem *item;
+	gint num;
+	MsgInfo *msginfo;
+	SummaryView *summaryview;
+
+	g_return_val_if_fail(target != NULL, -1);
+
+	debug_print("send_message_set_reply_flag(): "
+		    "setting reply flag to: %s\n", target);
+
+	summaryview = main_window_get()->summaryview;
+
+	item = folder_find_item_and_num_from_id(target, &num);
+	if (!item || num <= 0)
+		return -1;
+
+	if (summaryview->folder_item == item) {
+		msginfo = summary_get_msginfo_by_msgnum(summaryview, num);
+		if (msginfo) {
+			if (msgid && strcmp2(msgid, msginfo->msgid) != 0) {
+				debug_print("send_message_set_reply_flag(): "
+					    "message-id mismatch\n");
+				return -1;
+			}
+			MSG_UNSET_PERM_FLAGS(msginfo->flags, MSG_FORWARDED);
+			MSG_SET_PERM_FLAGS(msginfo->flags, MSG_REPLIED);
+			MSG_SET_TMP_FLAGS(msginfo->flags, MSG_FLAG_CHANGED);
+			if (MSG_IS_IMAP(msginfo->flags))
+				imap_msg_set_perm_flags(msginfo, MSG_REPLIED);
+			if (msginfo->folder)
+				msginfo->folder->mark_dirty = TRUE;
+			summary_update_by_msgnum(summaryview, msginfo->msgnum);
+                }
+	} else {
+		msginfo = procmsg_get_msginfo(item, num);
+		if (msginfo) {
+			if (msgid && strcmp2(msgid, msginfo->msgid) != 0) {
+				debug_print("send_message_set_reply_flag(): "
+					    "message-id mismatch\n");
+				procmsg_msginfo_free(msginfo);
+				return -1;
+			}
+			MSG_UNSET_PERM_FLAGS(msginfo->flags, MSG_FORWARDED);
+			MSG_SET_PERM_FLAGS(msginfo->flags, MSG_REPLIED);
+			MSG_SET_TMP_FLAGS(msginfo->flags, MSG_FLAG_CHANGED);
+			if (MSG_IS_IMAP(msginfo->flags))
+				imap_msg_set_perm_flags(msginfo, MSG_REPLIED);
+			if (msginfo->folder)
+				msginfo->folder->mark_dirty = TRUE;
+			procmsg_add_flags(msginfo->folder, msginfo->msgnum,
+					  msginfo->flags);
+			procmsg_msginfo_free(msginfo);
+                }
+	}
+
+	return 0;
+}
+
+gint send_message_set_forward_flags(const gchar *targets)
+{
+	FolderItem *item;
+	gint num;
+	MsgInfo *msginfo;
+	SummaryView *summaryview;
+	gchar **paths;
+	gint i;
+	GSList *mlist = NULL;
+
+	g_return_val_if_fail(targets != NULL, -1);
+
+	debug_print("send_message_set_forward_flags(): "
+		    "setting forward flags to: %s\n", targets);
+
+	summaryview = main_window_get()->summaryview;
+
+	paths = g_strsplit(targets, "\n", 0);
+
+	for (i = 0; paths[i] != NULL; i++) {
+		g_strstrip(paths[i]);
+
+		item = folder_find_item_and_num_from_id(paths[i], &num);
+		if (!item || num <= 0)
+			continue;
+
+		if (summaryview->folder_item == item) {
+			msginfo = summary_get_msginfo_by_msgnum(summaryview,
+								num);
+			if (msginfo) {
+				MSG_UNSET_PERM_FLAGS(msginfo->flags,
+						     MSG_REPLIED);
+				MSG_SET_PERM_FLAGS(msginfo->flags,
+						   MSG_FORWARDED);
+				MSG_SET_TMP_FLAGS(msginfo->flags,
+						  MSG_FLAG_CHANGED);
+				if (msginfo->folder)
+					msginfo->folder->mark_dirty = TRUE;
+				summary_update_by_msgnum
+					(summaryview, msginfo->msgnum);
+				msginfo = procmsg_msginfo_copy(msginfo);
+			}
+		} else {
+			msginfo = procmsg_get_msginfo(item, num);
+			if (msginfo) {
+				MSG_UNSET_PERM_FLAGS(msginfo->flags,
+						     MSG_REPLIED);
+				MSG_SET_PERM_FLAGS(msginfo->flags,
+						   MSG_FORWARDED);
+				MSG_SET_TMP_FLAGS(msginfo->flags,
+						  MSG_FLAG_CHANGED);
+				if (msginfo->folder)
+					msginfo->folder->mark_dirty = TRUE;
+				procmsg_add_flags(msginfo->folder,
+						  msginfo->msgnum,
+						  msginfo->flags);
+			}
+		}
+
+		if (msginfo)
+			mlist = g_slist_append(mlist, msginfo);
+	}
+
+	if (mlist) {
+		msginfo = (MsgInfo *)mlist->data;
+		if (MSG_IS_IMAP(msginfo->flags))
+			imap_msg_list_unset_perm_flags(mlist, MSG_REPLIED);
+	}
+
+	procmsg_msg_list_free(mlist);
+	g_strfreev(paths);
+
+	return 0;
+}
+
 static gint send_message_local(const gchar *command, FILE *fp)
 {
 	gchar **argv;
@@ -348,9 +590,11 @@ static gint send_message_local(const gchar *command, FILE *fp)
 
 	fd_close(child_stdin);
 
+#ifdef G_OS_UNIX
 	waitpid(pid, &status, 0);
 	if (!WIFEXITED(status) || WEXITSTATUS(status) != 0)
 		err = TRUE;
+#endif
 
 	g_spawn_close_pid(pid);
 
@@ -370,6 +614,7 @@ static gint send_message_smtp(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp)
 {
 	Session *session;
 	SMTPSession *smtp_session;
+	FILE *out_fp;
 	gushort port;
 	SendProgressDialog *dialog;
 	gchar buf[BUFFSIZE];
@@ -388,6 +633,8 @@ static gint send_message_smtp(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp)
 		ac_prefs->set_domain ? g_strdup(ac_prefs->domain) : NULL;
 
 	if (ac_prefs->use_smtp_auth) {
+		inc_lock();
+
 		smtp_session->forced_auth_type = ac_prefs->smtp_auth_type;
 
 		if (ac_prefs->smtp_userid) {
@@ -400,7 +647,7 @@ static gint send_message_smtp(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp)
 					g_strdup(ac_prefs->tmp_smtp_pass);
 			else {
 				smtp_session->pass =
-					input_dialog_query_password
+					input_query_password
 						(ac_prefs->smtp_server,
 						 smtp_session->user);
 				if (!smtp_session->pass)
@@ -417,7 +664,7 @@ static gint send_message_smtp(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp)
 					g_strdup(ac_prefs->tmp_pass);
 			else {
 				smtp_session->pass =
-					input_dialog_query_password
+					input_query_password
 						(ac_prefs->smtp_server,
 						 smtp_session->user);
 				if (!smtp_session->pass)
@@ -426,6 +673,8 @@ static gint send_message_smtp(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp)
 					g_strdup(smtp_session->pass);
 			}
 		}
+
+		inc_unlock();
 	} else {
 		smtp_session->user = NULL;
 		smtp_session->pass = NULL;
@@ -434,8 +683,18 @@ static gint send_message_smtp(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp)
 	smtp_session->from = g_strdup(ac_prefs->address);
 	smtp_session->to_list = to_list;
 	smtp_session->cur_to = to_list;
-	smtp_session->send_data = get_outgoing_rfc2822_str(fp);
-	smtp_session->send_data_len = strlen(smtp_session->send_data);
+
+	out_fp = get_outgoing_rfc2822_file(fp);
+	if (!out_fp) {
+		session_destroy(session);
+		return -1;
+	}
+	smtp_session->send_data_fp = out_fp;
+	smtp_session->send_data_len = get_left_file_size(out_fp);
+	if (smtp_session->send_data_len < 0) {
+		session_destroy(session);
+		return -1;
+	}
 
 #if USE_SSL
 	port = ac_prefs->set_smtpport ? ac_prefs->smtpport :
@@ -447,11 +706,18 @@ static gint send_message_smtp(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp)
 	port = ac_prefs->set_smtpport ? ac_prefs->smtpport : SMTP_PORT;
 #endif
 
+	if (ac_prefs->pop_before_smtp && ac_prefs->protocol == A_POP3) {
+		if (inc_pop_before_smtp(ac_prefs) < 0) {
+			session_destroy(session);
+			return -1;
+		}
+	}
+
 	dialog = send_progress_dialog_create();
 	dialog->session = session;
 
 	progress_dialog_append(dialog->dialog, NULL, ac_prefs->smtp_server,
-			       _("Connecting"), NULL);
+			       _("Connecting"), "", NULL);
 
 	g_snprintf(buf, sizeof(buf), _("Connecting to SMTP server: %s ..."),
 		   ac_prefs->smtp_server);
@@ -463,9 +729,17 @@ static gint send_message_smtp(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp)
 		(session, send_send_data_progressive, dialog);
 	session_set_send_data_notify(session, send_send_data_finished, dialog);
 
+	session_set_timeout(session, prefs_common.io_timeout_secs * 1000);
+
+	inc_lock();
+
 	if (session_connect(session, ac_prefs->smtp_server, port) < 0) {
+		manage_window_focus_in(dialog->dialog->window, NULL, NULL);
+		send_put_error(session);
+		manage_window_focus_out(dialog->dialog->window, NULL, NULL);
 		session_destroy(session);
 		send_progress_dialog_destroy(dialog);
+		inc_unlock();
 		return -1;
 	}
 
@@ -485,6 +759,11 @@ static gint send_message_smtp(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp)
 		/* consider EOF right after QUIT successful */
 		log_warning("%s\n", _("Connection closed by the remote host."));
 		ret = 0;
+	} else if (session->state == SESSION_ERROR &&
+		   SMTP_SESSION(session)->state == SMTP_QUIT) {
+		/* ignore errors right after QUIT */
+		log_warning("%s\n", _("Error occurred after QUIT command (ignored)"));
+		ret = 0;
 	} else if (session->state == SESSION_ERROR ||
 		   session->state == SESSION_EOF ||
 		   session->state == SESSION_TIMEOUT ||
@@ -502,6 +781,7 @@ static gint send_message_smtp(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp)
 
 	session_destroy(session);
 	send_progress_dialog_destroy(dialog);
+	inc_unlock();
 
 	return ret;
 }
@@ -580,6 +860,12 @@ static gint send_send_data_progressive(Session *session, guint cur_len,
 	progress_dialog_set_label(dialog->dialog, buf);
 	progress_dialog_set_percentage
 		(dialog->dialog, (gfloat)cur_len / (gfloat)total_len);
+	g_snprintf(buf, sizeof(buf), _("%d / %d bytes"),
+		   cur_len, total_len);
+	progress_dialog_set_row_progress(dialog->dialog, 0, buf);
+#ifdef G_OS_WIN32
+	GTK_EVENTS_FLUSH();
+#endif
 
 	return 0;
 }
@@ -634,6 +920,7 @@ static void send_cancel_button_cb(GtkWidget *widget, gpointer data)
 	SendProgressDialog *dialog = (SendProgressDialog *)data;
 
 	dialog->cancelled = TRUE;
+	session_disconnect(dialog->session);
 }
 
 static void send_put_error(Session *session)
@@ -666,9 +953,14 @@ static void send_put_error(Session *session)
 	default:
 		switch (session->state) {
 		case SESSION_ERROR:
-			log_msg =
-				_("Error occurred while sending the message.");
-			err_msg = g_strdup(log_msg);
+			if (SMTP_SESSION(session)->state == SMTP_READY) {
+				log_msg = _("Can't connect to SMTP server.");
+				err_msg = g_strdup_printf
+					(_("Can't connect to SMTP server: %s:%d"), session->server, session->port);
+			} else {
+				log_msg = _("Error occurred while sending the message.");
+				err_msg = g_strdup(log_msg);
+			}
 			break;
 		case SESSION_EOF:
 			log_msg = _("Connection closed by the remote host.");
